@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2018, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2020, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -287,7 +287,7 @@ typedef struct acpi_resource_irq
     UINT8                           DescriptorLength;
     UINT8                           Triggering;
     UINT8                           Polarity;
-    UINT8                           Sharable;
+    UINT8                           Shareable;
     UINT8                           WakeCapable;
     UINT8                           InterruptCount;
     UINT8                           Interrupts[1];
@@ -534,7 +534,7 @@ typedef struct acpi_resource_extended_irq
     UINT8                           ProducerConsumer;
     UINT8                           Triggering;
     UINT8                           Polarity;
-    UINT8                           Sharable;
+    UINT8                           Shareable;
     UINT8                           WakeCapable;
     UINT8                           InterruptCount;
     ACPI_RESOURCE_SOURCE            ResourceSource;
@@ -558,7 +558,7 @@ typedef struct acpi_resource_gpio
     UINT8                           ConnectionType;
     UINT8                           ProducerConsumer;   /* For values, see Producer/Consumer above */
     UINT8                           PinConfig;
-    UINT8                           Sharable;           /* For values, see Interrupt Attributes above */
+    UINT8                           Shareable;           /* For values, see Interrupt Attributes above */
     UINT8                           WakeCapable;        /* For values, see Interrupt Attributes above */
     UINT8                           IoRestriction;
     UINT8                           Triggering;         /* For values, see Interrupt Attributes above */
@@ -736,7 +736,7 @@ typedef struct acpi_resource_pin_function
 {
     UINT8                           RevisionId;
     UINT8                           PinConfig;
-    UINT8                           Sharable;           /* For values, see Interrupt Attributes above */
+    UINT8                           Shareable;           /* For values, see Interrupt Attributes above */
     UINT16                          FunctionNumber;
     UINT16                          PinTableLength;
     UINT16                          VendorLength;
@@ -750,7 +750,7 @@ typedef struct acpi_resource_pin_config
 {
     UINT8                           RevisionId;
     UINT8                           ProducerConsumer;   /* For values, see Producer/Consumer above */
-    UINT8                           Sharable;           /* For values, see Interrupt Attributes above */
+    UINT8                           Shareable;           /* For values, see Interrupt Attributes above */
     UINT8                           PinConfigType;
     UINT32                          PinConfigValue;
     UINT16                          PinTableLength;
@@ -794,7 +794,7 @@ typedef struct acpi_resource_pin_group_function
 {
     UINT8                           RevisionId;
     UINT8                           ProducerConsumer;   /* For values, see Producer/Consumer above */
-    UINT8                           Sharable;           /* For values, see Interrupt Attributes above */
+    UINT8                           Shareable;           /* For values, see Interrupt Attributes above */
     UINT16                          FunctionNumber;
     UINT16                          VendorLength;
     ACPI_RESOURCE_SOURCE            ResourceSource;
@@ -807,7 +807,7 @@ typedef struct acpi_resource_pin_group_config
 {
     UINT8                           RevisionId;
     UINT8                           ProducerConsumer;   /* For values, see Producer/Consumer above */
-    UINT8                           Sharable;           /* For values, see Interrupt Attributes above */
+    UINT8                           Shareable;           /* For values, see Interrupt Attributes above */
     UINT8                           PinConfigType;      /* For values, see PinConfigType above */
     UINT32                          PinConfigValue;
     UINT16                          VendorLength;
